﻿<#
	Restarts SPAdminV4 and SPTimerV4 service on all servers.
#>
function Restart-SPServices($parameters, [System.Xml.XmlElement]$data, [string]$extId, [string]$extensionPath) {
    Log -message ("Extension ID: "+ $extId) -type $SPSD.LogTypes.Normal
    Log -message ("Extension path: "+ $extensionPath) -type $SPSD.LogTypes.Normal

    $serviceNames = @("SPAdminV4", "SPTimerV4")

    Log -message "Retrieving services" -type $SPSD.LogTypes.Normal
    $services = (Get-SPFarm).Services |? { $_.Name -in $serviceNames }

    $services |% {
        Log -message ("Stopping service instances of " + $_) -type $SPSD.LogTypes.Normal
        $_.Instances |% {
            Log -message ("Stopping service instance on " + $_.Server.Address) -type $SPSD.LogTypes.Normal
            
            try {
                $_.Stop()
            } catch {
                Log -message ("Failed to stop service: " + $_.Exception.Message) -type $SPSD.LogTypes.Error
            }
        }
    }

    Log -message "Waiting until all services are stopped" -type $SPSD.LogTypes.Normal
    sleep 10

    $services |% {
        Log -message ("Starting service instances of " + $_) -type $SPSD.LogTypes.Normal
        $_.Instances |% {
            Log -message ("Starting service instance on " + $_.Server.Address) -type $SPSD.LogTypes.Normal
            
            try {
                $_.Start()
            } catch {
                Log -message ("Failed to start service: " + $_.Exception.Message) -type $SPSD.LogTypes.Error
            }
        }
    }

	sleep 5
}