﻿<#
 # # # # # # #
 # .SYNOPSIS
 #      Installs the specified license
 # # # # # # #
 #
 # .PARAMETER $ManagerAssembly
 #      The name of the assembly that contains the license manager class.
 # .PARAMETER $ManagerClass
 #      The name of the license manager class including namespace.
 # .PARAMETER LicensePath
 #      Specifies the license path.
 #>
param
(
	[parameter(Mandatory=$true, ValueFromPipeline=$false)] [string] $ManagerAssembly,
	[parameter(Mandatory=$true, ValueFromPipeline=$false)] [string] $ManagerClass,
    [parameter(Mandatory=$false, ValueFromPipeline=$false)] [string] $LicensePath
)

try
{
	if (-not $LicensePath)
	{
		$LicensePath = $PSScriptRoot.ToString().TrimEnd("\") + "\..\license.xml"
	}

	Write-Host "License: $LicensePath"

	if (-not (Test-Path $LicensePath))
	{
		Write-Host "License file could not be found!" -f Red
		pause
	}
	else
	{
		[System.Reflection.Assembly]::LoadWithPartialName($ManagerAssembly) | Out-Null
		$ls = new-object $ManagerClass
		$ls.ImportLicenseFromFile($LicensePath)
	}
}
catch
{
    Write-Host -Fore Red "$_ $StackTrace"
	pause
}